#include "AutomatonPrinter.h"

AutomatonPrinter::AutomatonPrinter(std::ostream& out) : m_Out(out) {

}

void AutomatonPrinter::printStateSymbolTo(automaton::FSM& automaton, automaton::State from, alphabet::Symbol input) {
  bool sign = false;
  for(auto iterTransitions = automaton.getTransitions().begin(); iterTransitions != automaton.getTransitions().end(); iterTransitions++) {
    if(iterTransitions->getFrom() == from && iterTransitions->getInput() == input) {
      m_Out << (sign ? "|" : " ") << iterTransitions->getTo().getName();
      sign = true;
    }
  }
  if(!sign) {
    m_Out << " -";
  }
}

void AutomatonPrinter::print(automaton::FSM& automaton) {
  
  if(automaton.isDeterministic()) {
    m_Out << "DFA";
  } else {
    m_Out << "NFA";
  }
  
  for(auto iterSymbol = automaton.getInputAlphabet().begin(); iterSymbol != automaton.getInputAlphabet().end(); iterSymbol++) {
    m_Out << " " << iterSymbol->getSymbol();
  }
  
  bool isEpsilonFreeSign = automaton.isEpsilonFree();
  if(!isEpsilonFreeSign) {
    m_Out << " \\eps";
  }
  
  m_Out << std::endl;
  
  for(auto iterState = automaton.getStates().begin(); iterState != automaton.getStates().end(); iterState++) {
    if(automaton.getInitialStates().find(*iterState) != automaton.getInitialStates().end()) {
      m_Out << ">";
    }
    if(automaton.getFinalStates().find(*iterState) != automaton.getFinalStates().end()) {
      m_Out << "<";
    }
    m_Out << iterState->getName();
    
    for(auto iterSymbol = automaton.getInputAlphabet().begin(); iterSymbol != automaton.getInputAlphabet().end(); iterSymbol++) {
      printStateSymbolTo(automaton, *iterState, *iterSymbol);
    }

    if(!isEpsilonFreeSign) {
      printStateSymbolTo(automaton, *iterState, alphabet::Symbol(""));
    }
    
    m_Out << std::endl;
  }
  
}

