#include <iostream>
#include <fstream>
#include "AutomatonParser.h"
#include "AutomatonPrinter.h"

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#include "automaton/FSM/FSM.h"
#include "automaton/UnknownAutomaton.h"
#include "AutomatonFactory.h"
#include "AlibException.h"

#define FROM 1
#define TO 2

int fromAutomaton(std::istream& in, std::ostream& out) {
  AutomatonParser parser(in);
  
  automaton::FSM result = parser.parse();
  
  out << result;
  
  return 0;
}

int toAutomaton(std::istream& in, std::ostream& out) {
  
  try {

    automaton::UnknownAutomaton automaton = automaton::AutomatonFactory::fromStream(in);
    if(!automaton::AutomatonFactory::isFSM(automaton)) {
      std::cerr << "Automaton minimize require deterministic finite automaton\n" << std::endl;
      return 1;
    }
    automaton::FSM fsm = automaton::AutomatonFactory::buildFSM(automaton);
    AutomatonPrinter printer = AutomatonPrinter(out);
    printer.print(fsm);

  } catch (alib::AlibException& e) {
    std::cerr << e.getCause() << std::endl;
    return 1;
  }
  return 0;
}

int main(int argc, char* argv[]) {

  int fromTo = 0;
  int c;
  
  while (1) {
    static struct option long_options[] = {
      {"from", no_argument,       &fromTo, FROM},
      {"to",   no_argument,       &fromTo, TO},
      {0, 0, 0, 0}
    };
  
    int option_index = 0;
     
    c = getopt_long (argc, argv, "", long_options, &option_index);

    if (c == -1)
      break;
  
    switch (c) {
    case 0:
      break;
    default:
      optind--;
      goto more;
    }
  }
more:

  std::istream* in = &std::cin;
  if(optind != argc) in = new std::ifstream(argv[optind]);

  int res;
  if(fromTo == FROM) {
    res = fromAutomaton(*in, std::cout);
  } else if(fromTo == TO) {
    res = toAutomaton(*in, std::cout);
  } else {
    std::cerr << "error" << std::endl;
    res = 1;
  }
  
  if(c != -1) delete in;
  return res;
}