#include "DeterminizationUtils.h"

namespace determinization {

string DeterminizationUtils::buildStateName(const set<State>& states) {
    string name = "[";
    for (set<State>::const_iterator state = states.begin(); state != states.end(); state++) {
        if (state != states.begin()) {
            name += ", ";
        }
        name += "'" + state->getName() + "'";
    }
    name += "]";
    return name;
}


const State& DeterminizationUtils::getOrCreateState(const set<State>& originalStates, map<string, StateData>& states, Automaton& automaton) {
    string stateName = buildStateName(originalStates);
    
    map<string, StateData>::iterator stateIter = states.find(stateName);
    if (stateIter != states.end()) {
        return stateIter->second.state;
    }
    
    State state(stateName);
    StateData stateData(state, originalStates);
    StateData& insertedData = states.insert(pair<string, StateData>(stateName, stateData)).first->second;
    automaton.addState(state);
    return insertedData.state;
}


void DeterminizationUtils::copyInputAlphabet(const Automaton& fromAutomaton, Automaton& toAutomaton) {
    const set<Symbol>& alphabet = fromAutomaton.getInputAlphabet();
    for (set<Symbol>::iterator input = alphabet.begin(); input != alphabet.end(); input++) {
        toAutomaton.addInputSymbol(*input);
    }
}


void DeterminizationUtils::copyStackAlphabet(const PDA& fromPDA, PDA& toPDA) {
    const set<Symbol>& alphabet = fromPDA.getStackAlphabet();
    for (set<Symbol>::iterator symbol = alphabet.begin(); symbol != alphabet.end(); symbol++) {
        toPDA.addStackSymbol(*symbol);
    }
}


bool DeterminizationUtils::containSomeFinalStateOfAutomaton(const set<State>& states, const Automaton& automaton) {
    const set<State>& finalStates = automaton.getFinalStates();
    set<State> intersection;
    set_intersection(finalStates.begin(), finalStates.end(), states.begin(), states.end(),
            std::inserter(intersection, intersection.begin()));
    return intersection.size() > 0;
}

} /* namespace determinization */
