/*
 * Transition.h
 *
 *  Created on: Apr 16, 2013
 *      Author: martin
 */

#ifndef TRANSITION_H_
#define TRANSITION_H_

#include "State.h"
#include "../alphabet/Symbol.h"

namespace automaton {

using namespace alphabet;

/**
 * Abstract base class for all transitions. Contains common elements of transitions.
 */
class Transition {
protected:
	State from;
	Symbol input;
	State to;
public:
	Transition(const State& current, const Symbol& input, const State& next);
	virtual ~Transition();

	/**
	 * @param state State from which the transition goes
	 */
	void setFrom(const State& state);

	/**
	 * @return State form which the transition goes
	 */
	const State& getFrom() const;

	/**
	 * @param symbol input symbol for this transition
	 */
	void setInput(const Symbol& symbol);

	/**
	 * @return input symbol for this transition
	 */
	const Symbol& getInput() const;

	/**
	 * @param state State to which the transition goes
	 */
	void setTo(const State& state);

	/**
	 * @return State to which the transition goes
	 */
	const State& getTo() const;

	/**
	 * Determines whether State is used in this transition either as from state or to state.
	 * @return true when transition contains the state, false otherwise
	 */
	bool containsState(const State& state) const;

	friend std::ostream& operator<<(std::ostream&, const Transition&);

	virtual std::ostream& operator>>(std::ostream&) const;
};

} /* namespace automaton */
#endif /* TRANSITION_H_ */
