/*
 * GrammarParser.h
 *
 *  Created on: Nov 3, 2013
 *      Author: martin
 */

#ifndef GRAMMARPARSER_H_
#define GRAMMARPARSER_H_

#include <list>
#include <string>
#include "UnknownGrammar.h"
#include "Rule.h"
#include "../sax/Token.h"

namespace grammar {

using namespace std;
using namespace sax;

/**
 * Parser used to get UnknownGrammar from XML parsed into list of Tokens.
 */
class GrammarParser {
protected:
	static void parseNonTerminalSymbols(list<Token>& input, Grammar& grammar);
	static void parseTerminalSymbols(list<Token>& input, Grammar& grammar);
	static void parseRules(list<Token>& input, Grammar& grammar);
	static Rule parseRule(list<Token>& input);

	static void parseStartSymbol(list<Token>& input, Grammar& grammar);

	static Symbol parseSymbol(list<Token> &input, string tagName = "symbol");
	static list<Symbol> parseSymbolList(list<Token>& input, string tagName);

	static bool isToken(list<Token> &input, Token::TokenType type, string data);
	static void popToken(list<Token> &input, Token::TokenType type, string data);
public:
	/**
	 * Parses the Token list and returns the grammar. The input is destroyed in the process.
	 * @param input XML represented as list of Tokens
	 * @return UnknownGrammar
	 * @throws ParserException when an error occurs
	 */
	static UnknownGrammar parse(list<Token>& input);
};

} /* namespace grammar */
#endif /* GRAMMARPARSER_H_ */
