/*
 * RightLinearGrammar.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#include "RightLinearGrammar.h"

namespace grammar {

RightLinearGrammar::RightLinearGrammar() :
		LinearGrammar() {
}

RightLinearGrammar::RightLinearGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols,
		const Symbol& startSymbol) :
		LinearGrammar(nonTerminalSymbols, terminalSymbols, startSymbol) {
}

bool RightLinearGrammar::isValidRule(const Rule& rule) const {
	return checkLeftSide(rule.getLeftSide()) && checkRightSide(rule.getRightSide());
}

bool RightLinearGrammar::checkLeftSide(const list<Symbol>& leftSide) const {
	if (leftSide.size() != 1) {
		return false;
	}

	if (nonTerminalSymbols.find(leftSide.front()) == nonTerminalSymbols.end()) {
		return false;
	}

	return true;
}

bool RightLinearGrammar::checkRightSide(const list<Symbol>& rightSide) const {
	if (rightSide.size() == 0) {
		return true;
	} else if (rightSide.size() == 1) {
		const Symbol& symbol = rightSide.front();

		//check epsilon
		if (symbol.getSymbol() == "") {
			return true;
		}

		//check that symbol exists
		return ((terminalSymbols.find(symbol) != terminalSymbols.end())
				|| (nonTerminalSymbols.find(symbol) != nonTerminalSymbols.end()));

	} else {
		//check if all symbols except the last one are terminal symbols
		for (list<Symbol>::const_iterator symbol = rightSide.begin(); symbol != --rightSide.end(); symbol++) {
			if (terminalSymbols.find(*symbol) == terminalSymbols.end()) {
				return false;
			}
		}

		//check if last symbol is nonterminal
		const Symbol& lastSymbol = *(--rightSide.end());
		if (nonTerminalSymbols.find(lastSymbol) == nonTerminalSymbols.end()) {
			return false;
		}

		return true;
	}
}

} /* namespace grammar */
