/*
 * LeftRegularGrammar.cpp
 *
 *  Created on: Nov 17, 2013
 *      Author: martin
 */

#include "LeftRegularGrammar.h"

namespace grammar {

LeftRegularGrammar::LeftRegularGrammar() :
		RegularGrammar() {
}

LeftRegularGrammar::LeftRegularGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols,
		const Symbol& startSymbol) :
		RegularGrammar(nonTerminalSymbols, terminalSymbols, startSymbol) {
}

bool LeftRegularGrammar::isValidRule(const Rule& rule) const {
	if (isEpsilonRule(rule)) {
		return checkEpsilonRule(rule.getLeftSide(), rule.getRightSide());
	} else {
		return checkLeftSide(rule.getLeftSide()) && checkRightSide(rule.getRightSide());
	}
}

bool LeftRegularGrammar::checkLeftSide(const list<Symbol>& leftSide) const {
	if (leftSide.size() != 1) {
		return false;
	}

	if (nonTerminalSymbols.find(leftSide.front()) == nonTerminalSymbols.end()) {
		return false;
	}

	return true;
}

bool LeftRegularGrammar::checkRightSide(const list<Symbol>& rightSide) const {
	if (rightSide.size() == 1) {
		return (terminalSymbols.find(rightSide.front()) != terminalSymbols.end());
	} else if (rightSide.size() == 2) {

		//cannot add rule that contains start symbol on the right side when grammar already contains epsilon rule
		if (rightSide.front() == startSymbol && containsEpsilonRule()) {
			return false;
		}

		//check if first symbol is nonterminal
		if (nonTerminalSymbols.find(rightSide.front()) == nonTerminalSymbols.end()) {
			return false;
		}

		//check if second symbol is terminal
		if (terminalSymbols.find(rightSide.back()) == terminalSymbols.end()) {
			return false;
		}
		return true;

	} else {
		return false;
	}
}

} /* namespace grammar */
