/*
 * RightRegularGrammar.h
 *
 *  Created on: Nov 10, 2013
 *      Author: martin
 */

#ifndef RIGHTREGULARGRAMMAR_H_
#define RIGHTREGULARGRAMMAR_H_

#include "RegularGrammar.h"

namespace grammar {

/**
 * Right regular grammar. Produces regular language.
 */
class RightRegularGrammar: public RegularGrammar {
private:
	/**
	 * Checks that left side contains one symbol which is nonterminal.
	 * @param leftSide left side of the Rule
	 * @return true when left side contains one symbol which is nonterminal, false otherwise
	 */
	bool checkLeftSide(const list<Symbol>& leftSide) const;

	/**
	 * Checks that right side of the rule contains one terminal symbol
	 * or terminal symbol followed by nonterminal symbol. Epsilon rules are checked
	 * in RegularGrammar class.
	 * @param rightSide right side of the Rule
	 * @return true when right side of the rule is valid, false otherwise
	 */
	bool checkRightSide(const list<Symbol>& rightSide) const;
protected:
	/**
	 * @copydoc Grammar::isValidRule(const Rule&) const
	 */
	bool isValidRule(const Rule& rule) const;
public:
	RightRegularGrammar();
	RightRegularGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols,
			const Symbol& startSymbol);
};

} /* namespace grammar */
#endif /* RIGHTREGULARGRAMMAR_H_ */
