/*
 * Concatenation.cpp
 *
 *  Created on: Nov 27, 2013
 *      Author: martin
 */

#include "Concatenation.h"

namespace regexp {

Concatenation::Concatenation() {
}

Concatenation::Concatenation(const Concatenation& other) {
	for (auto element : other.elements) {
		elements.push_back(element->clone());
	}
}

Concatenation& Concatenation::operator =(const Concatenation& other) {
	if(this == &other) {
		return *this;
	}

	for (auto element : elements) {
		delete element;
	}
	elements.clear();

	for (auto element : other.elements) {
		elements.push_back(element->clone());
	}

	return *this;
}

Concatenation::~Concatenation() {
	for (auto element : elements) {
		delete element;
	}
	elements.clear();
}

list<RegExpElement*>& Concatenation::getElements() {
	return elements;
}

const list<RegExpElement*>& Concatenation::getElements() const {
	return elements;
}

RegExpElement* Concatenation::clone() const {
	return new Concatenation(*this);
}

bool Concatenation::operator<(const RegExpElement& other) const {
	return other > *this;
}

bool Concatenation::operator==(const RegExpElement& other) const {
	return other == *this;
}

bool Concatenation::operator>(const RegExpElement& other) const {
	return other < *this;
}


bool Concatenation::operator<(const Concatenation& other) const {
	int thisSize = this->elements.size();
	int otherSize = other.elements.size();
	if(thisSize < otherSize) return true;
	if(thisSize > otherSize) return false;

	auto thisIter = this->elements.begin();
	auto otherIter = other.elements.begin();
	for(; thisIter != this->elements.end(); thisIter++, otherIter++) {
	  if(**thisIter < **otherIter) return true;
	}

	return false;
}

bool Concatenation::operator==(const Concatenation& other) const {
	if(this->elements.size() != other.elements.size()) return false;

	auto thisIter = this->elements.begin();
	auto otherIter = other.elements.begin();
	for(; thisIter != this->elements.end(); thisIter++, otherIter++) {
	  if(**thisIter != **otherIter) return false;
	}

	return true;
}

bool Concatenation::containsEmptyString() const {
	for( const auto& e : getElements())
		if( ! e->containsEmptyString())
			return false;

	return true;
}

} /* namespace regexp */
