/*
 * RegExpElement.h
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#ifndef REGEXPELEMENT_H_
#define REGEXPELEMENT_H_

namespace regexp {

using namespace std;


class Alternation;
class Concatenation;
class Iteration;
class RegExpSymbol;
class RegExpEmpty;
class RegExpEpsilon;

/**
 * Abstract class representing element in the regular expression. Can be operator or symbol.
 */
class RegExpElement {
public:
	virtual ~RegExpElement();

	/**
	 * Creates copy of the element.
	 * @return copy of the element
	 */
	virtual RegExpElement* clone() const = 0;

	virtual bool operator<(const RegExpElement&) const = 0;
	virtual bool operator==(const RegExpElement&) const = 0;
	virtual bool operator>(const RegExpElement&) const = 0;

	virtual bool operator>=(const RegExpElement&) const;
	virtual bool operator<=(const RegExpElement&) const;
	virtual bool operator!=(const RegExpElement&) const;

	virtual bool operator<(const Concatenation&) const;
	virtual bool operator<(const Alternation&) const;
	virtual bool operator<(const Iteration&) const;
	virtual bool operator<(const RegExpSymbol&) const;
	virtual bool operator<(const RegExpEpsilon&) const;
	virtual bool operator<(const RegExpEmpty&) const;

	virtual bool operator==(const Concatenation&) const;
	virtual bool operator==(const Alternation&) const;
	virtual bool operator==(const Iteration&) const;
	virtual bool operator==(const RegExpSymbol&) const;
	virtual bool operator==(const RegExpEpsilon&) const;
	virtual bool operator==(const RegExpEmpty&) const;

	/**
	 * @return true if this subtree of regexp matches empty string (epsilon)
	 */
	virtual bool containsEmptyString() const = 0;
};

} /* namespace regexp */
#endif /* REGEXPELEMENT_H_ */
