//============================================================================
// Name        : aminimize.cpp
// Author      : Jan Travnicek
//============================================================================

#include <iostream>
#include <string>
#include <set>

#include "AutomatonFactory.h"
#include "AlibException.h"
#include "automaton/AutomatonParser.h"

#include "Normalize.h"

using namespace std;
using namespace automaton;
using namespace alib;

int main(int argc, char** argv) {

	UnknownAutomaton automaton;

	try {

		if (argc == 2 && string("-h").compare(argv[1]) == 0) {
			cout << "Automaton normalize.\nUsage: anormalize automaton.xml\n";
			return -1;
		} else if (argc == 1 || (argc == 2 && string("--").compare(argv[1]) == 0)) {
			automaton = AutomatonFactory::fromStdin();
			if(!AutomatonFactory::isFSM(automaton)) {
				cout << "Automaton normalize require minimal deterministic finite automaton\n";
				return 1;
			}
		} else if (argc == 2) {
			automaton = AutomatonFactory::fromFile(argv[1]);
			if(!AutomatonFactory::isFSM(automaton)) {
				cout << "Automaton normalize require minimal deterministic finite automaton\n";
				return 1;
			}
		} else {
			cout << "Automaton normalize require finite automaton\n";
			return 1;
		}
		automaton::FSM fsm = AutomatonFactory::buildFSM(automaton);
		if(!fsm.isDeterministic()) {
			cout << "Automaton normalize require minimal deterministic finite automaton\n";
			return 1;
		}
			
		automaton::FSM res = Normalize::normalize(fsm);
		res.toXML(cout);

		return 0;

	} catch (AlibException& e) {
		cout << e.getCause() << endl;
		return 0;
	}

}
