/*
 * RedundantStateRemover.cpp
 *
 *  Created on: 4. 3. 2014
 *      Author: tomas
 */

#include "RedundantStateRemover.h"

using namespace automaton;
using namespace std;

namespace trim
{

FSM RedundantStateRemover::remove( const FSM & fsm )
{
    deque<set<State>> Qi;
    Qi.push_back( set<State>( ) );

    int i = 1;
    for( const auto & f : fsm.getFinalStates( ) )
        Qi.at( 0 ).insert( f );

    while( true )
    {
        Qi.push_back( Qi.at( i - 1 ) ); // copy Qi-1 into Qi
        for( const auto & p : Qi.at( i - 1 ) )
            for( const auto & t : fsm.getTransitionsToState( p ) )
                Qi.at( i ).insert( t.getFrom( ) );

        if( Qi.at( i ) == Qi.at( i - 1 ) )
            break;

        i += 1;
    }

    FSM ret;

    for( const auto & q : Qi.at( i ) )
        ret.addState( q );

    for( const auto & t : fsm.getTransitions( ) )
    {
        if( isInSet( t.getFrom( ), ret.getStates( ) ) && isInSet( t.getTo( ), ret.getStates( ) ) )
        {
            if( ! isInSet( t.getInput( ), ret.getInputAlphabet( ) ) )
                ret.addInputSymbol( t.getInput( ) );

            ret.addTransition( t );
        }
    }

    for( const auto & q : fsm.getInitialStates( ) )
        ret.addInitialState( q );
    for( const auto & q : fsm.getFinalStates( ) )
        ret.addFinalState( q );

    return ret;
}

} /* namespace trim */
